﻿using System;
using System.Collections.Generic;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Input;
using UniversalEditor.Base;
using UniversalEditor.Base.FileSystem;

namespace TextEditor
{
	public abstract class TextEditorBase : EditorBase
	{
		protected readonly Lazy<UfeTextBox> _textBox;

		protected TextEditorBase(IEditorOwner owner, FileWrapper file, Lazy<UfeTextBox> textBox)
			: base(owner, file)
		{
			if (textBox == null) 
				throw new ArgumentNullException("textBox");
			
			_textBox = textBox;
		}

		public override FrameworkElement EditorControl
		{
			get { return _textBox.Value; }
		}

		public override void Reload()
		{
			if (_textBox.IsValueCreated)
			{
				_textBox.Value.Document.Reload();
				_textBox.Value.CallOnRender();
			}
		}

		public override bool HasChanges
		{
			get { return _file.HasChanges; }
		}

		public override void Focus()
		{
			_textBox.Value.TextBox.UpdateLayout();
			_textBox.Value.TextBox.Focus();
		}

		public override void SaveFile()
		{
			if (_textBox.IsValueCreated)
				_textBox.Value.Document.Save(_file.FilePath);
		}

		public override void SaveFileAs(string filePath)
		{
			_textBox.Value.Document.Save(filePath);
		}

		public override void SaveCopyAs(string filePath)
		{
			_textBox.Value.Document.SaveCopy(filePath);
		}

		public override IList<MenuItem> EditorCommands
		{
			get { return _textBox.Value.GenerateBasicMenu(); }
		}

		public override void OnKeyDown(KeyEventArgs keyEventArgs)
		{
			_textBox.Value.TextBox.OnKeyDownEngine(keyEventArgs);
			base.OnKeyDown(keyEventArgs);
		}

		public override void OnTextInput(TextCompositionEventArgs textCompositionEventArgs)
		{
			_textBox.Value.TextBox.OnTextInputEngine(textCompositionEventArgs);
			base.OnTextInput(textCompositionEventArgs);
		}

		public override bool HasEditorCommands
		{
			get { return true; }
		}
	}
}
