﻿using System;
using System.IO;
using System.Xml.Linq;

namespace UniversalEditor.Base.Options
{
	public class Options
	{
		private static Options _instance;

		public static Options Instance
		{
			get
			{
				if (_instance == null)
				{
					_instance = Load();
				}

				return _instance;
			}
		}

		private string _fontName = "Consolas";
		private double _fontSize = 14;
		private bool _isStrongSearch = false;

		public bool IsStrongSearch
		{
			get { return _isStrongSearch; }
			set { _isStrongSearch = value; }
		}

		public string FontName
		{
			get { return _fontName; }
			set { _fontName = value; }
		}

		public double FontSize
		{
			get { return _fontSize; }
			set { _fontSize = value; }
		}

		private static Options Load()
		{
			string optionsDir = Path.Combine(Environment.GetFolderPath(Environment.SpecialFolder.UserProfile), "UniversalFileEditor");
			string optionsFile = Path.Combine(optionsDir, "options.xml");

			if (!File.Exists(optionsFile))
				return new Options();

			Options options = new Options();
			XDocument doc = XDocument.Load(optionsFile);

			XElement fontNameElement = doc.Root.Element("FontName");
			if (fontNameElement != null)
				options.FontName = fontNameElement.Value;

			XElement fontSizeElement = doc.Root.Element("FontSize");
			if (fontSizeElement != null)
				options.FontSize = double.Parse(fontSizeElement.Value);

			XElement isStrongSearchElement = doc.Root.Element("IsStrongSearch");
			if (isStrongSearchElement != null)
				options.IsStrongSearch = bool.Parse(isStrongSearchElement.Value);

			return options;
		}

		public void Save()
		{
			string optionsDir = Path.Combine(Environment.GetFolderPath(Environment.SpecialFolder.UserProfile), "UniversalFileEditor");
			string optionsFile = Path.Combine(optionsDir, "options.xml");

			if (!Directory.Exists(optionsDir))
				Directory.CreateDirectory(optionsDir);

			XDocument document = new XDocument(
				new XElement("Options",
					new XElement("FontName", FontName),
					new XElement("FontSize", FontSize),
					new XElement("IsStrongSearch", IsStrongSearch)));
			
			document.Save(optionsFile);
		}
	}
}
