﻿using System.Data;
using System.IO;
using System.Windows;
using System.Windows.Controls;
using UniversalEditor.Base;
using UniversalEditor.Base.FileSystem;

namespace UniversalEditor.Csv
{
	public class CsvEditor : EditorBase
    {
		private readonly Border _border;
		private readonly DataGrid _grid;

		public CsvEditor(IEditorOwner owner, FileWrapper file)
			: base(owner, file)
		{
			_file.ContentChanged += OnFileContentChanged;

			_border = new Border();
			
			// initializing
			_grid = new DataGrid();
			_grid.IsReadOnly = true;
			_grid.ItemsSource = GetDataView();

			_border.Child = _grid;
		}
		public override void Dispose()
		{
			_file.ContentChanged -= OnFileContentChanged;

			base.Dispose();
		}

		public override void Focus()
		{
			base.Focus();
		}

		public override void Reload()
		{
			_grid.ItemsSource = GetDataView();
		}

		public override FrameworkElement EditorControl
	    {
		    get { return _border; }
	    }
		
		private DataView GetDataView()
		{
			DataTable table = new DataTable();

			using (TextReader reader = new StreamReader(File.OpenRead(_file.FilePath)))
			{
				string line;

				while ((line = reader.ReadLine()) != null)
				{
					DataRow row = table.Rows.Add();
					string[] data = line.Split(';');

					while (data.Length > table.Columns.Count)
						table.Columns.Add();

					for (int index = 0; index < data.Length; index++)
					{
						row[index] = data[index];
					}
				}
			}

			return table.AsDataView();
		}

		private void OnFileContentChanged()
		{
			Reload();
		}
    }
}
