﻿using System;
using System.Diagnostics;
using System.Windows;
using System.Windows.Input;
using System.Windows.Media;

namespace UniversalEditor.Video.Viewer
{
	/// <summary>
	/// Interaction logic for VideoViewerView.xaml
	/// </summary>
	public partial class VideoViewerView
	{
		private bool _isPlaying;

		public VideoViewerView()
		{
			InitializeComponent();
		}

		internal void OpenFile(string filePath)
		{
			media.Source = new Uri(filePath);
			
			media.Play();
			_isPlaying = true;
		}
		
		private void OnMediaOpened(object sender, RoutedEventArgs e)
		{
			progress.Minimum = 0;
			progress.Maximum = media.NaturalDuration.TimeSpan.TotalSeconds;

			volume.Value = media.Volume*100d;
		}

		private void OnPlayClick(object sender, RoutedEventArgs e)
		{
			media.Play();
			_isPlaying = true;
		}

		private void OnPauseClick(object sender, RoutedEventArgs e)
		{
			media.Pause();
			_isPlaying = false;
		}

		private void OnStopClick(object sender, RoutedEventArgs e)
		{
			media.Stop();
			_isPlaying = false;
		}

		private void OnVolumeChanged(object sender, RoutedPropertyChangedEventArgs<double> e)
		{
			media.Volume = e.NewValue/100d;
		}

		private void OnMediaFailed(object sender, ExceptionRoutedEventArgs e)
		{
			Debug.Fail(e.ErrorException.Message);
		}

		private void OnMouseUp(object sender, MouseButtonEventArgs e)
		{
			if (_isPlaying)
			{
				media.Pause();
				_isPlaying = false;
			}
			else
			{
				media.Play();
				_isPlaying = true;
			}
		}
	}
}
