﻿using System.Collections.Generic;
using TextEditor.TextBoxInner.DrawingElements;
using TextEditor.TextBoxInner.Utils;

namespace TextEditor.TextBoxInner.Layers
{
	internal class TextLayer : LayerBase<TextLine>
	{
		private static readonly TextLineCreator _textLineCreator = new TextLineCreator();
		
		private readonly List<TextLine> _tempInvisibleList = new List<TextLine>(50);
		private readonly List<TextLine> _tempCandidateseList = new List<TextLine>(50);

		public TextLayer() : base(_textLineCreator)
		{ }

		public void SetLines(RenderedInfo[] lines)
		{
#if DEBUG
			//System.Diagnostics.Debug.Assert(lines.Length < Children.Count);
#endif

			try
			{
				if (lines.Length == 0)
				{
					for (int index = 0; index < Children.Count; index++)
						Children[index].SetContent(null);

					return;
				}

				_tempInvisibleList.Clear();
				_tempCandidateseList.Clear();

				int startIndex = lines[0].LineIndex;
				int endIndex = lines[lines.Length - 1].LineIndex;

				int startCandidatesIndex = -1;
//				int endCandidatesIndex = -1;

				for (int index = 0; index < Children.Count; index++)
				{
					TextLine line = Children[index];

					if (!line.IsVisible)
					{
						_tempInvisibleList.Add(line);
						continue;
					}

					if (line.LineIndex < startIndex)
					{
						_tempInvisibleList.Add(line);
						continue;
					}

					if (line.LineIndex > endIndex)
					{
						_tempInvisibleList.Add(line);
						continue;
					}

					_tempCandidateseList.Add(line);

//					endCandidatesIndex = line.LineIndex;

					if (startCandidatesIndex < 0)
						startCandidatesIndex = line.LineIndex;
				}

				for (int index = 0; index < lines.Length; index++)
				{
					RenderedInfo item = lines[index];
					int itemIndex = item.LineIndex;

					if (itemIndex == startCandidatesIndex)
					{
						for (int marker = 0; marker < _tempCandidateseList.Count; marker++)
							Children[index + marker] = _tempCandidateseList[marker];

						index += _tempCandidateseList.Count - 1;
					}
					else
					{
						int tempElementIndex = _tempInvisibleList.Count - 1;
						TextLine temp = _tempInvisibleList[tempElementIndex];
						_tempInvisibleList.RemoveAt(tempElementIndex);

						Children[index] = temp;
						temp.SetContent(item);
					}
				}

				for (int index = lines.Length; index < Children.Count; index++)
				{
					int tempElementIndex = _tempInvisibleList.Count - 1;
					TextLine temp = _tempInvisibleList[tempElementIndex];
					_tempInvisibleList.RemoveAt(tempElementIndex);

					Children[index] = temp;
					temp.SetContent(null);
				}

				for (int index = 0; index < Children.Count; index++)
					Children[index].SetTop(index * FontHelper.FontHeight);
			}
			finally
			{
				InvalidateArrange();
			}
		}

		private class TextLineCreator : DrawingElementFactory<TextLine>
		{
			internal override TextLine Create()
			{
				return new TextLine();
			}
		}
	}
}
