﻿using System;
using System.Collections.Generic;
using System.Windows;
using System.Windows.Controls;
using UniversalEditor.Base;
using UniversalEditor.Base.FileSystem;
using UniversalEditor.Base.MenuBarItems;
using UniversalEditor.Base.Mvvm;

namespace UniversalEditor.Archive
{
	public class ArchiveEditor : EditorBase
	{
		private readonly Lazy<ArchiveModel> _model;
		private readonly Lazy<ArchiveViewModel> _viewModel;
		private readonly Lazy<ArchiveView> _view;

		public ArchiveEditor(IEditorOwner owner, FileWrapper file)
			: base(owner, file)
		{
			_model = new Lazy<ArchiveModel>(CreateModel);
			_viewModel = new Lazy<ArchiveViewModel>(CreateViewModel);
			_view = new Lazy<ArchiveView>(CreateView);
		}

		private ArchiveViewModel CreateViewModel()
		{
			return new ArchiveViewModel(_model.Value);
		}

		private ArchiveView CreateView()
		{
			return new ArchiveView { DataContext = _viewModel.Value };
		}

		private ArchiveModel CreateModel()
		{
			return new ArchiveModel(_file);
		}

		public override MenuListItem Menu1
		{
			get
			{
				return new MenuListItem("Editor", null, new MenuItemBase[]
				{
					new MenuCommandItem("Extract Selected Items...", null, null, _viewModel.Value.ExtractCommand), 
					new MenuCommandItem("Extract All...", null, null, _viewModel.Value.ExtractAllCommand)
				});
			}
		}

		public override void Dispose()
		{
			if (_model.IsValueCreated)
				_model.Value.Dispose();

			base.Dispose();
		}

		public override FrameworkElement EditorControl
		{
			get { return _view.Value; }
		}
    }
}
